indexing
	description
		"XalanTransformer interface, reuses compiled style sheets & parsed source."
		"Any parameters must be reset before repeated transform operations."

class
	XALAN_TRANSFORMER

inherit
	XML
	CL_MISC
	DISPOSABLE

create
	make

feature -- Creation

	make is

		do
			xml.initialize
			obj := new_transformer
		end

	dispose is

		do
			delete_output
			if obj /= default_pointer then
				delete_transformer (obj)
				obj := default_pointer
			end
		end

feature -- Access

	error: BOOLEAN

	set_quoted_parameter (key, expression: STRING) is
			-- String expressions should be enclosed in single quotes.
		do
			set_parameter (key, "'" + expression + "'")
		end

	set_parameter (key, expression: STRING) is
			--
		do
			parameter (obj, terminated (key), terminated (expression))
		end

	set_indent (indent: INTEGER) is
			-- String expressions should be enclosed in single quotes.
		do
			xt_set_indent (obj, indent)
		end

	set_use_validation (flag: BOOLEAN) is

		local
			i: INTEGER
		do
			if flag then i := 1 end
			validation (obj, i)
		end

	set_in_file (file_name: STRING) is
			-- Specify the input XML file.
		do
			xt_in := parse_file (obj, terminated (file_name))
		end

	set_in_string (xml_data: STRING) is
			-- Specify the input XML file.
		do
			xt_in := parse_string (obj, str_to_ptr (xml_data), xml_data.count)
		end

	set_in_xerces (xerces: DOM_DOCUMENT) is
			-- Specify the input XML file.
		do
			xt_in := dom_wrapper (obj, xerces.obj)
		end

	parse_error: BOOLEAN is
			--
		do
			Result := xt_in = default_pointer
		end

	set_xslt_file (file_name: STRING) is

		do
			css := compile_stylesheet (obj, terminated (file_name))
		end

	delete_output is
			--
		do
			if ss /= default_pointer then
				delete_stringstream (ss)
				ss := default_pointer
			end
			if (xt_out /= default_pointer) and (dst_mode /= Dst_xerces) then
				delete_target (xt_out)
				xt_out := default_pointer
			end
		end

	set_out_string is
			--
		do
			delete_output
			ss := string_stream
			xt_out := string_target (ss)
			dst_mode := Dst_string
		ensure
			stream_exists
		end

	stream_exists: BOOLEAN is
			--
		do
			Result := ss /= default_pointer
		end

	set_out_file (file_name: STRING) is
			-- Send the output XML to a file.
		do
			delete_output
			xt_out := file_target (terminated (file_name))
			dst_mode := Dst_file
		end

	set_out_xerces is
			--
		do
			delete_output
			create output_document.make (Void, Void)
			xt_out := output_document.obj
			dst_mode := Dst_xerces
		end

	transform is

		do
			rv := transform_xml (dst_mode, obj, xt_in, css, xt_out)
			error := rv /= 0
		end

	output: STRING is

		require
			stream_exists
		local
			len: INTEGER
		do
			len := stream_length (ss)
			create Result.make_filled (' ', len)
			copy_stream (ss, str_to_ptr (Result), len)
		end

	output_document: DOM_DOCUMENT

	test (xml_fname, xsl_fname: STRING): INTEGER is

		do
			Result:= feature_test (obj, terminated (xml_fname), terminated (xsl_fname))
		end

feature {} -- External

-- transform modes

	Src_file: INTEGER is 1
	Src_string: INTEGER is 2
	Src_xerces: INTEGER is 4
	Dst_file: INTEGER is 8
	Dst_string: INTEGER is 16
	Dst_xerces: INTEGER is 32
	Xsl_file: INTEGER is 64
	Xsl_css: INTEGER is 128

	obj, css, ss, xt_in, xt_out: POINTER -- parsed source
	src_mode, dst_mode: INTEGER

	new_transformer: POINTER is

		external
			"C | %"xalan_support.h%""
		end

	delete_transformer (t: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

	parameter (t, key, expression: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

	xt_set_indent (t: POINTER; indent: INTEGER) is

		external
			"C | %"xalan_support.h%""
		end

	validation (t: POINTER; flag: INTEGER) is

		external
			"C | %"xalan_support.h%""
		end

	feature_test (t, xml_fname, xsl_fname: POINTER): INTEGER is

		external
			"C | %"xalan_support.h%""
		end

	compile_stylesheet (t, fname: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	parse_file (t, fname: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	parse_string (t, buffer: POINTER; size: INTEGER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	dom_wrapper (t, dom: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	string_stream: POINTER is

		external
			"C | %"xalan_support.h%""
		end

	string_target (stream: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	file_target (fname: POINTER): POINTER is

		external
			"C | %"xalan_support.h%""
		end

	delete_stringstream (stream: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

	delete_target (target: POINTER) is

		external
			"C | %"xalan_support.h%""
		end

	transform_xml (mode: INTEGER; t, parsed_source, compiled_stylesheet, res: POINTER): INTEGER is

		external
			"C | %"xalan_support.h%""
		end

	stream_length (stream: POINTER): INTEGER is

		external
			"C | %"xalan_support.h%""
		end

	copy_stream (stream, strdata: POINTER; length: INTEGER) is

		external
			"C | %"xalan_support.h%""
		end

end
